//////////////////////////////////////////////////////////////////////
//	LibMaster.com NTFSpath library - ntfspath.h
//	COPYRIGHT (C) 2006-2007 LibMaster.com.  All Rights Reserved.
//////////////////////////////////////////////////////////////////////
// ntfspath.h : Intrface of NTFSpath library
//////////////////////////////////////////////////////////////////////
//
/** \file ntfspath.h Intrface of NTFSpath library */
/*
 * The following ifdef block is the standard way of creating macros which make exporting
 * from a DLL simpler. All files within this DLL are compiled with the NTFSPATH_EXPORTS
 * symbol defined on the command line. this symbol should not be defined on any project
 * that uses this DLL. This way any other project whose source files include this file see
 * NTFSPATH_API functions as being imported from a DLL, whereas this DLL sees symbols
 * defined with this macro as being exported.
 */

#if !defined(NTFSPATH_H__66C2542C_CBE1_11D6_9360_0020182E0D04__INCLUDED_)
#define NTFSPATH_H__66C2542C_CBE1_11D6_9360_0020182E0D04__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <tchar.h>
#include <windows.h>

#ifdef __cplusplus
extern "C"
{
#endif


#if !defined UNICODE || !defined _UNICODE
	#error NTFSpath library can be used with UNICODE only! Please define UNICODE
#endif

#ifndef NTFSPATH_STATIC
	#ifdef NTFSPATH_EXPORTS
	#define NTFSPATH_API __declspec(dllexport)
	#else
	#define NTFSPATH_API __declspec(dllimport)
	#endif
#else
	#define NTFSPATH_API
#endif


/*! Error codes */

typedef enum _err_codes {
	NO_ERRORS,
	INVALID_PATH,
	BUFFER_IS_TOO_SMALL,
	UNDEFINED_ERROR,
	FILE_NOT_FOUND
} err_codes;

#define NTFS_PATH_SIZE 0xFFFF
#define NTFS_PATH_LEN NTFS_PATH_SIZE/sizeof(TCHAR)

/*! Returns TRUE when the path string begins from "\\?\" prefix */
NTFSPATH_API BOOL has_NTFS_path_prefix(const TCHAR * path);

/** Returns TRUE when the path string begins from "\\?\UNC\" or "\\" prefixes */
NTFSPATH_API BOOL is_UNC_path(const TCHAR * path);

/**Remove \\ or \\?\UNC\ prefix from the input path */
NTFSPATH_API BOOL rem_UNC_path_prefix(const TCHAR * path, TCHAR * buff, size_t len);

/**Remove \\?\ prefix from the input path */
NTFSPATH_API BOOL rem_NTFS_path_prefix(const TCHAR * path, TCHAR * buff, size_t len);

/**Add \\?\ prefix to the input path if it is not exist */
NTFSPATH_API BOOL add_NTFS_path_prefix(const TCHAR * path, TCHAR * buff, size_t len);

/**
	Returns a part of the input path after the last slash ('\' or '/')
	or NULL when no one slashes were not found.
*/
NTFSPATH_API const TCHAR * get_file_part(const TCHAR * path);

/**	Returns a part of the input path after the first slash ('\' or '/')
	or NULL when no one slashes were not found.
	NTFS prefix (\\?\) or UNC prefic ("\\?\UNC\") are ignored */
NTFSPATH_API const TCHAR * get_path_part(const TCHAR * path);

/** Find a colon in the path. If a symbol before the colon is alpha it returns
a pointer to the symbol in other cases it returns NULL */
NTFSPATH_API const TCHAR * get_drive_part(const TCHAR * path);

/** When an any function in the library returns FALSE it set an error code.
You can get this error code by using of get_err_code() finction.
You can get a description of an error code by using of get_error_text() finction.*/
NTFSPATH_API err_codes get_err_code();

/** Gets a string with an error description by an error code */
NTFSPATH_API const TCHAR * get_error_text(err_codes errc);

/**If the drive part is exist in the path it is absolute path */
NTFSPATH_API BOOL is_absolute(const TCHAR * path);

/**If the path string contains ? or * then this path is a mask */
NTFSPATH_API BOOL is_mask(const TCHAR * path);

/** Remove a part of the path after last slash.
in case path and buff point to the same address
a part of the path after last slash just removed without copying */
NTFSPATH_API BOOL path_pop(const TCHAR * path, TCHAR * buff, size_t len);

/** Add a slash and the input string to the input path 
in case path and buff point to the same address
a slash and the input string just added without copying */
NTFSPATH_API BOOL path_push(const TCHAR * path, const TCHAR * str, TCHAR * buff, size_t len);

/** Get attributes of the file */
NTFSPATH_API BOOL get_file_attr(const TCHAR * path, WIN32_FILE_ATTRIBUTE_DATA * attr);

/** ret is TRUE when file is exist */
NTFSPATH_API BOOL file_exist(const TCHAR * path, BOOL * ret);

/**ret is true when file is exist and it is directory */
NTFSPATH_API BOOL is_directory(const TCHAR * path, BOOL * ret);

/** 
	Implements an advanced comparison algorithm.
	It supports comparison of absolute and relative paths, 
	masks in the path and some other.
*/
NTFSPATH_API BOOL path_mask_cmp(const TCHAR* path, const TCHAR* mask);

#ifdef __cplusplus
};
#endif

#endif // !defined(NTFSPATH_H__66C2542C_CBE1_11D6_9360_0020182E0D04__INCLUDED_)
