//////////////////////////////////////////////////////////////////////
//	LibMaster.com NTFSpath library - test.c
//	COPYRIGHT (C) 2006-2007 LibMaster.com.  All Rights Reserved.
//////////////////////////////////////////////////////////////////////
// test.c : Test application for functions using demonstration
//              and regression testing purposes.
//////////////////////////////////////////////////////////////////////
//

#include <tchar.h>
#include "ntfspath.h"

TCHAR * BOOLtoStr(BOOL v) {
	return v?_T("TRUE"):_T("FALSE");
}

int has_NTFS_path_prefix_test() {

	TCHAR * prefix = _T("\\\\?\\c:\\This string have NTFS path prefix");
	TCHAR * no_prefix = _T("c:\\This string have no NTFS path prefix");

	BOOL res1, res2;

	_tprintf(_T("test of function: [BOOL has_NTFS_path_prefix(const TCHAR * path)]\n"));

	/*! Returns TRUE when the path string begins from "\\?\" prefix */
	
	 res1 = has_NTFS_path_prefix(prefix);
	_tprintf(_T("%s == %s.\n"), prefix, BOOLtoStr(res1));

	 res2 = has_NTFS_path_prefix(no_prefix);
	_tprintf(_T("%s == %s.\n"), no_prefix, BOOLtoStr(!res2));

	return (res1 && (!res2));
}

int is_UNC_path_test() {

	TCHAR * prefix1 = _T("\\\\?\\UNC\\c:\\This\\string/have UNC path prefix\\");
	TCHAR * prefix2 = _T("\\\\This\\string/have/UNC/path\\prefix\\also");
	TCHAR * no_prefix = _T("c:\\This string have no UNC path prefix");

	BOOL res1, res2, res3;

	/** Returns TRUE when the path string begins from "\\?\UNC\" or "\\" prefixes */
	_tprintf(_T("test of function: [BOOL is_UNC_path(const TCHAR * path)]\n"));
	
	 res1 = is_UNC_path(prefix1);
	_tprintf(_T("%s == %s.\n"), prefix1, BOOLtoStr(res1));

	 res2 = is_UNC_path(prefix2);
	_tprintf(_T("%s == %s.\n"), prefix2, BOOLtoStr(res2));

	 res3 = is_UNC_path(no_prefix);
	_tprintf(_T("%s == %s.\n"), no_prefix, BOOLtoStr(!res3));

	return (res1 && res2 && (!res3));
}

int rem_NTFS_path_prefix_test() {

	TCHAR * path = _T("\\\\?\\c:\\This string have NTFS path prefix");
	TCHAR buff[NTFS_PATH_LEN];
	BOOL res;

	_tprintf(_T("test of function: [BOOL rem_NTFS_path_prefix(const TCHAR * path, TCHAR * buff, size_t len)]\n"));

	res = rem_NTFS_path_prefix(path, buff, NTFS_PATH_LEN);

	res = (! has_NTFS_path_prefix(buff)) && res;

	if(res) {
		_tprintf(_T("%s == FALSE.\n"), buff);
	}

	return res;
}

int add_NTFS_path_prefix_test() {
	TCHAR * path = _T("c:\\This string have no NTFS path prefix");
	TCHAR buff[NTFS_PATH_LEN];
	BOOL res;

	_tprintf(_T("test of function: [BOOL add_NTFS_path_prefix(const TCHAR * path, TCHAR * buff, size_t len)]\n"));

	/**Add \\?\ prefix to the input path */
	res = add_NTFS_path_prefix(path, buff, NTFS_PATH_LEN);

	res = has_NTFS_path_prefix(buff) && res;
	if(res) {
		_tprintf(_T("%s == FALSE.\n"), buff);
	}

	return res;
}

int get_file_part_test() {
	TCHAR * path = _T("c:\\This\\is/the\\path___/this_is_the_file_part.txt");
	const TCHAR * file_part = path;
//	BOOL res;

	_tprintf(_T("test of function: [const TCHAR * get_file_part(const TCHAR * path)]\n"));

	/**
		Returns a part of the input path after the last slash ('\' or '/')
		or NULL when no one slashes were not found.
	*/

	file_part = get_file_part(path);
	_tprintf(_T("Path: %s\n"), path);
	if((file_part != NULL) && (_tcscmp(file_part, _T("this_is_the_file_part.txt")) == 0)) {
		_tprintf(_T("File part of the path: %s\n"), file_part);
		return TRUE;
	}
	return FALSE;
}

int get_path_part_test() {
	TCHAR * path = _T("\\\\?\\c:\\This\\is/the\\path___/this_is_the_file_part.txt");
	const TCHAR * path_part = path;
//	BOOL res;

	_tprintf(_T("test of function: [const TCHAR * get_path_part(const TCHAR * path)]\n"));

	/**	Returns a part of the input path after the first slash ('\' or '/')
		or NULL when no one slashes were not found.
	NTFS prefix (\\?\) or UNC prefic ("\\?\UNC\") are ignored */

	path_part = get_path_part(path);
	_tprintf(_T("full path: %s\n"), path);
	if((path_part != NULL) && (_tcscmp(path_part, _T("This\\is/the\\path___/this_is_the_file_part.txt")) == 0)) {
		_tprintf(_T("Path part of the full path: %s\n"), path_part);
		return TRUE;
	}
	return FALSE;	
}

int get_drive_part_test() {
	TCHAR * path = _T("\\\\?\\c:\\This\\is/the\\path___/this_is_the_file_part.txt");
	const TCHAR * drive_part = path;
//	BOOL res;

	_tprintf(_T("test of function: [const TCHAR * get_drive_part(const TCHAR * path)]\n"));

	/** Find a colon in the path. If a symbol before the colon is alpha it returns
	a pointer to the symbol in other cases it returns NULL */

	drive_part = get_drive_part(path);
	_tprintf(_T("full path: %s\n"), path);
	if((drive_part != NULL) && (_tcsncmp(drive_part, _T("c:"),2) == 0)) {
		_tprintf(_T("Drive part of the full path: %s\n"), drive_part);
		return TRUE;
	}
	return FALSE;	
}

int is_absolute_test() {
	TCHAR * abs_path = _T("\\\\?\\c:\\This\\is/the\\absolute/path.txt");
	TCHAR * rel_path = _T("../c\\this_is_the_relative_path.txt");
	BOOL res1, res2;

	_tprintf(_T("test of function: [BOOL is_absolute(const TCHAR * path)]\n"));

	/**If the drive part is exist in the path it is absolute path */
	res1 = is_absolute(abs_path);
	_tprintf(_T("%s == %s.\n"), abs_path, BOOLtoStr(res1));

	res2 = is_absolute(rel_path);
	_tprintf(_T("%s == %s.\n"), rel_path, BOOLtoStr(!res2));

	return res1 && (!res2);

}


int is_mask_test() {
	TCHAR * mask = _T("../c\\this_is_the_mask?.*");
	TCHAR * not_mask = _T("../c\\this_is_not_a_mask.txt");
	BOOL res, res2;

	_tprintf(_T("test of function: [BOOL is_absolute(const TCHAR * path)]\n"));

	/**If the path string contains ? or * then this path is a mask */

	res = is_mask(mask);
	_tprintf(_T("%s == %s.\n"), mask, BOOLtoStr(res));

	res2 = is_mask(not_mask);
	_tprintf(_T("%s == %s.\n"), not_mask, BOOLtoStr(!res2));

	return res && (!res2);
}

int path_pop_test() {
	TCHAR * path = _T("../c\\this\\is\\the/path/this_part_must_be_removed.txt");
	TCHAR buff[NTFS_PATH_LEN] = {0};
//	BOOL res;

	/** Remove a part of the path after last slash */
	_tprintf(_T("test of function: [BOOL path_pop(const TCHAR * path, TCHAR * buff, size_t len)]\n"));

	path_pop(path, buff, NTFS_PATH_LEN);

	return _tcscmp(buff, _T("../c\\this\\is\\the/path")) == 0;
}

int path_push_test() {
	TCHAR * path = _T("../c\\this\\is\\the/path");
	TCHAR * str = _T("this_part_must_be_added.txt");
	TCHAR buff[NTFS_PATH_LEN];
//	BOOL res;

	/** Add a slash and the input string to the input path */
	_tprintf(_T("test of function: [BOOL path_push(const TCHAR * path, const TCHAR * str, TCHAR * buff, size_t len)]\n"));

	path_push(path, str, buff, NTFS_PATH_LEN);

	return _tcscmp(buff, _T("../c\\this\\is\\the/path\\this_part_must_be_added.txt")) == 0;
}

void print_file_attrs(const TCHAR * file, WIN32_FILE_ATTRIBUTE_DATA * attr) {

	_tprintf(_T("The file [%s] has following attributes:\n"), file);

	if(attr->dwFileAttributes & FILE_ATTRIBUTE_READONLY           ) { _tprintf(_T("READONLY           \n")); }
	if(attr->dwFileAttributes & FILE_ATTRIBUTE_HIDDEN             ) { _tprintf(_T("HIDDEN             \n")); }
	if(attr->dwFileAttributes & FILE_ATTRIBUTE_SYSTEM             ) { _tprintf(_T("SYSTEM             \n")); }
	if(attr->dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY          ) { _tprintf(_T("DIRECTORY          \n")); }
	if(attr->dwFileAttributes & FILE_ATTRIBUTE_ARCHIVE            ) { _tprintf(_T("ARCHIVE            \n")); }
	if(attr->dwFileAttributes & FILE_ATTRIBUTE_DEVICE             ) { _tprintf(_T("DEVICE             \n")); }
	if(attr->dwFileAttributes & FILE_ATTRIBUTE_NORMAL             ) { _tprintf(_T("NORMAL             \n")); }
	if(attr->dwFileAttributes & FILE_ATTRIBUTE_TEMPORARY          ) { _tprintf(_T("TEMPORARY          \n")); }
	if(attr->dwFileAttributes & FILE_ATTRIBUTE_SPARSE_FILE        ) { _tprintf(_T("SPARSE_FILE        \n")); }
	if(attr->dwFileAttributes & FILE_ATTRIBUTE_REPARSE_POINT      ) { _tprintf(_T("REPARSE_POINT      \n")); }
	if(attr->dwFileAttributes & FILE_ATTRIBUTE_COMPRESSED         ) { _tprintf(_T("COMPRESSED         \n")); }
	if(attr->dwFileAttributes & FILE_ATTRIBUTE_OFFLINE            ) { _tprintf(_T("OFFLINE            \n")); }
	if(attr->dwFileAttributes & FILE_ATTRIBUTE_NOT_CONTENT_INDEXED) { _tprintf(_T("NOT_CONTENT_INDEXED\n")); }
	if(attr->dwFileAttributes & FILE_ATTRIBUTE_ENCRYPTED          ) { _tprintf(_T("ENCRYPTED          \n")); }
}


int get_file_attr_test() {

	WIN32_FILE_ATTRIBUTE_DATA attr;

	/** Get attributes of the file */
	_tprintf(_T("test of function: [BOOL get_file_attr(const TCHAR * path, WIN32_FILE_ATTRIBUTE_DATA * attr)]\n"));
	
	get_file_attr(_T(".\\test.c"), &attr);
	print_file_attrs(_T(".\\test.c"), &attr);

	return TRUE;
}
int file_exist_test() {
	BOOL res;
	/** ret is TRUE when file is exist */
	_tprintf(_T("test of function: [BOOL file_exist(const TCHAR * path, BOOL * ret)]\n"));
	file_exist(_T("test.c"), &res);
	return res;
}

int is_directory_test() {
	BOOL res1, res2;
	/**ret is true when file is exist and it is directory */
	_tprintf(_T("test of function: [BOOL is_directory(const TCHAR * path, BOOL * ret)]\n"));
	is_directory(_T(".\\test.c"), &res1);
	is_directory(_T("."), &res2);
	return (!res1) && res2;
}

int _tmain(int argc, TCHAR* argv[])
{

	typedef int (*fun)();

	fun func_arr[] = {
		has_NTFS_path_prefix_test,
		is_UNC_path_test,
		rem_NTFS_path_prefix_test,
		add_NTFS_path_prefix_test,
		get_file_part_test,
		get_path_part_test,
		get_drive_part_test,
		is_absolute_test,
		is_mask_test,
		path_pop_test,
		path_push_test,
		get_file_attr_test,
		file_exist_test,
		is_directory_test,
		NULL
	};

	int i = 0;
	int res = FALSE;
	int passed = 0, failed = 0;

	for(i = 0; func_arr[i] != NULL; i++) {
		res = (func_arr[i])();
		_tprintf(_T("\nTesting result is %s\n\n"), BOOLtoStr(res));
		if(!res) {
			_tprintf(_T("\n Error: %s\n\n"), get_error_text(get_err_code())); 
			failed++;
		}
		else {
			passed++;
		}
	}

	_tprintf(_T("\nSuccessfully passed tests: %d\n"), passed);
	_tprintf(_T("Failed tests: %d\n"), failed);

	return 0;
}
